import json
import re
import requests
import constants

def buildCollectiveInsightsPayload(self, advanced_params) -> json:
    """
    Parse the input parameters and format into a Collective Insights json object.
    Also tags the ThreatConnect entites with a Collective Insights label.
    """
    result = {}
    data = []
    
    collective_insights_payload = {}
    ioc_payload = constants.BASE_IOC_TEMPLATE.copy()

    # Advanced parameters setup
    options = { "summary": bool(True) }
    organization_ids = []

    # parse associated groups
    mitre_codes = [parse_mitre_codes(x["value"]) for x in self.groups if x["type"] == "Attack Pattern"]
    malwares = [x["value"] for x in self.groups if x["type"] == "Malware"]
    cves = [x for x in self.groups if x["type"] == "Vulnerability"]

    if mitre_codes:
        ioc_payload["mitre_codes"] = mitre_codes
    if malwares:
        ioc_payload["malwares"] = malwares
    if cves:
        self.indicators.extend(cves)

    if advanced_params:
        self.tcex.log.info("Parsing advanced parameters.")

        if "debug" in advanced_params:
            options["debug"] = parse_options(advanced_params["debug"])

        if "organization_ids" in advanced_params:
            organization_ids = parse_advanced(advanced_params["organization_ids"])
            
        if "timestamp" in advanced_params:
            ioc_payload["timestamp"] = advanced_params["timestamp"]
            
        if "source_type" in advanced_params:
            ioc_payload["ioc"]["source_type"] = advanced_params["source_type"]
            
        if "field" in advanced_params:
            ioc_payload["ioc"]["field"] = advanced_params["field"]
            
        if "incident_id" in advanced_params:
            ioc_payload["incident"]["id"] = advanced_params["incident_id"]
        if "incident_name" in advanced_params:
            ioc_payload["incident"]["name"] = advanced_params["incident_name"]
        if "incident_type" in advanced_params:
            ioc_payload["incident"]["type"] = advanced_params["incident_type"]

        detection = {}
        if "detection_type" in advanced_params:
            ioc_payload["detection"]["type"] = advanced_params["detection_type"]
        if "detection_subtype" in advanced_params:
            ioc_payload["detection"]["sub_type"] = advanced_params["detection_subtype"]
        if "detection_name" in advanced_params:
            ioc_payload["detection"]["name"] = advanced_params["detection_name"]
        if "detection_id" in advanced_params:
            ioc_payload["detection"]["id"] = advanced_params["detection_id"]

    if self.has_incident:
        if "id" not in ioc_payload["incident"]:
            ioc_payload["incident"]["id"] = self.uuid
        if "name" not in ioc_payload["incident"]:
            ioc_payload["incident"]["name"] = self.incident_name
        if "name" not in ioc_payload["detection"]:
            ioc_payload["detection"]["name"] = self.incident_name
        
        group_type = self.incident["type"]
        group_id = self.incident["id"]
        ti = self.tcex.ti.group(group_type=group_type, unique_id=group_id)
        ti.add_tag(constants.COLLECTIVE_INSIGHTS_TAG)
        response = ti.update()

    self.tcex.log.info("Parsing IOCs.")
    for indicator in self.indicators:
        indicator_type = mapIndicatorType(indicator["type"])
        
        if indicator_type not in constants.INDICATOR_TYPES:
            self.tcex.log.warning(f"Indicator type not supported for Collective Insights: {indicator_type}.")
            continue

        
        current_ioc_payload = ioc_payload.copy()
        indicator_value = indicator["value"]
        #indicator_owner = indicator["ownerName"]
        tc_indicator_type = indicator["type"]
        indicator_id = indicator["id"]

        # Get indicator to update Collective Insights tag
        if tc_indicator_type == "Vulnerability":
            ti = ti = self.tcex.ti.group(group_type=indicator_type, unique_id=indicator_id)
        else:
            #ti = self.tcex.ti.indicator(indicator_type=tc_indicator_type, owner=indicator_owner, unique_id=indicator_value)
            ti = self.tcex.ti.indicator(indicator_type=tc_indicator_type, unique_id=indicator_value)
        tag_response = ti.add_tag(name=constants.COLLECTIVE_INSIGHTS_TAG)
        ti.update()

        if current_ioc_payload["ioc"]: 
            # case if payload already has advanced params: source_type, field
            current_ioc_payload["ioc"]["type"] = indicator_type
            current_ioc_payload["ioc"]["value"] = indicator_value
        else:
            current_ioc = {
                "type": indicator_type,
                "value": indicator_value
            }
            current_ioc_payload["ioc"] = current_ioc

        data.append(current_ioc_payload)
    
    # Finalize result payload
    result["data"] = data
    
    result["options"] = options
    if organization_ids:
        result["organization_ids"] = organization_ids

    return json.dumps(result)


def submitCollectiveInsights(self, collective_insights_upload) -> json:
    """
    Posts the packaged json object to the Collective Insights endpoint.
    """

    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "X-RFToken": self.token,
        "User-Agent": f"{constants.APP_ID}/{constants.APP_VERSION}"
    }
    
    data = collective_insights_upload
    self.tcex.log.debug(data)

    # post data to collective insights using arguments created above
    response = requests.post(self.url, headers=headers, data=data)
    return response.json()


# Helper functions
def parse_options(advanced_option) -> bool:
    """
    Format input parameter from input to boolean type.
    """

    if advanced_option.lower() == "true":
        return bool(True)
    else:
        return bool(False)


def parse_mitre_codes(mitre_codes) -> list:
    """
    Regex function returning Mitre T-Codes from Attack Pattern Group input.
    """

    pattern = "T[0-9][.[0-9]+"
    return re.findall(pattern, mitre_codes)[0]


def parse_advanced(advanced_param):
    """
    Helper function to handle string and stringArray types for advanced input.
    """

    if isinstance(advanced_param, str):
        return [x.strip() for x in advanced_param.split(",")]
    return advanced_param


def mapIndicatorType(ioc_type) -> str:
    """
    Translates ThreatConnect to Recorded Future API naming conventions (if needed).
    """

    if ioc_type == "Address":
        return "ip"
    elif ioc_type == "Host":
        return "domain"
    elif ioc_type == "File":
        return "hash"
    elif ioc_type in ["URL", "Vulnerability"]:
        return ioc_type.lower()
    else:
        return ioc_type